package com.takensoft.taken_bi_manager.common.connection.api.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.takensoft.taken_bi_manager.common.connection.api.vo.ConnectionApi;
import com.takensoft.taken_bi_manager.common.util.CommonUtil;
import com.takensoft.taken_bi_manager.common.util.HTTPUtil;

/**
 * @author 김성원
 * @since 2024.01.12
 * 
 * API 연계 관련 기능 리스트 입니다. 
 */
public class ApiUtil {

	
	/**
	 * @author 김성원
	 * @since 2024.01.12
	 * 
	 * OpenApi 데이터 수집 기능 
	 */
	public static ConnectionApi apiRequest(ConnectionApi connApi) throws Exception {
		ObjectMapper mapper = new ObjectMapper();

		HTTPUtil httpcon = new HTTPUtil();	
		connApi.setResult(httpcon.HttpConnectionApi(connApi.getUrl(),connApi.getRequestType(), connApi.getParams()));
	
        return connApi;
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.12
	 * 
	 * OpenApi 데이터 수집 기능 
	 */
	public static List<Map<String,Object>>  getApiDataList (ConnectionApi connApi) throws Exception{
		ObjectMapper mapper = new ObjectMapper();

		// depth 리스트
		String [] depthList = connApi.getDepth().split(">");
		CommonUtil commonUtil = new CommonUtil();

		// Return Type :  1=JSON, 2=XML
		if(connApi.getType() ==2) {
			// XML String to JSON String
			String jsonStr = commonUtil.xmlStrToJsonStr(connApi.getResult());
			connApi.setResult(jsonStr);
		}
		
		JSONParser parser = new JSONParser();		
		
		JSONObject jsonObject = null;
		JSONArray jsonArray = null;
		
		if(connApi.getResult().substring(0,1).equals("[")) {
			jsonArray = (JSONArray) parser.parse(connApi.getResult());
		}else {
			jsonObject = (JSONObject) parser.parse(connApi.getResult());
		}
		
		List<Map<String, Object>> apiDataList = new ArrayList<Map<String, Object>>();
		
		if(jsonArray == null) {
			// jsonObject to Map
			Map<String, Object> resultMap = new HashMap<String, Object>();
			resultMap = commonUtil.jsonObjectToMap(jsonObject);
		
			int deptCount = 0;			
			for(String depth : depthList) {
				deptCount++;
				depth = depth.trim();			
				Object temp = (Object)resultMap.get(depth);
				
				// 리스트 판별
				if(temp instanceof List) {
					if(deptCount < depthList.length) {
						List<Map<String, Object>> tempList = (List<Map<String, Object>>) temp;
						resultMap =  tempList.get(0);
					}else {
						apiDataList = (List<Map<String, Object>>) temp;
					}		
				}else {
					resultMap = (Map<String, Object>) temp;	
				}
			}
		}else {
			apiDataList = (List<Map<String, Object>>)jsonArray;
		}
		
		int max = 0;
		int indx = 0;
		
		// 최대 길이의 키값을 가진 객체 구하기 		
		for(int i = 0 ; i < apiDataList.size() ; i++){
			if(apiDataList.get(i).keySet().size() > max) {
				max = apiDataList.get(i).keySet().size();
				indx = i;
			}			
		}
		
		// 빈정보 공백으로 채우기 
		for(Map<String, Object> item : apiDataList ) {
			if(max != item.size()) {
				for (Map.Entry<String, Object> entry : apiDataList.get(indx).entrySet()) 
				{
					if(item.get(entry.getKey()) == null) {
						item.put(entry.getKey(),"");
					}
				}
			}
		}		
		
		return apiDataList;
	}
}
