package com.takensoft.taken_bi_manager.common.connection.db.dao;



import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.takensoft.taken_bi_manager.common.connection.db.vo.ConnectionDB;
import com.takensoft.taken_bi_manager.common.connection.db.vo.TableBasicInfo;
import com.takensoft.taken_bi_manager.data.vo.ColumnData;
import com.takensoft.taken_bi_manager.data.vo.DataTable;
import com.takensoft.taken_bi_manager.data.vo.Dataset;


/**
 * @author 김성원
 * @since 2024.01.09
 * 
 * 데이터베이스 접속 모듈에 관련된 SQL문에 접근하는 DAO 추상 Class입니다.
 */
public abstract class DatabaseDAO {

	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * SqlSessionTemplate 생성 메서드 입니다.
	 */
	public abstract void createConnector (ConnectionDB db) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 접속한 데이터베이스의 테이블 목록 조회 SQL에 접근하는 메소드 입니다.
	 */
	public abstract List<Dataset> getDBConnectionTableList(ConnectionDB db) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커넥션된 데이터베이스의 테이블의 컬럼 정보 조회 (DB 기준 데이터 타입, 데이터 크기 등)
	 */
	public abstract List<ColumnData> getDBConnectionTableColumnList (Dataset dataset) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 특정 테이블의 데이터 목록 조회 SQL에 접근하는 메소드 입니다.
	 */
	public abstract List<LinkedHashMap<String, Object>> getDBConnectionTableDataList (DataTable datatable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 특정 테이블의 데이터 총 개수 조회 SQL에 접근하는 메소드 입니다.
	 */
	public abstract int getDBConnectionTableDataTotalRows (DataTable datatable) throws Exception;
	
		
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 특정 테이블의 데이터 목록 조회 SQL에 접근하는 메소드 입니다.
	 */
	public abstract List<LinkedHashMap<String, Object>> getDBConnectionCustomQueryDataList (DataTable datatable) throws Exception;
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 조회 없는 실행형 쿼리 
	 */
	public abstract int executeCustomQuery (DataTable datatable) throws Exception;
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.097
	 * 
	 * 특정 테이블의 데이터 총 개수 조회 SQL에 접근하는 메소드 입니다.
	 */
	public abstract int getDBConnectionCustomQueryDataTotalRows (DataTable datatable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * DB연계 데이터 목록 조회 (초기 DB화 때에 등록한 컬럼만 가지고 옴)
	 */
	public abstract List<LinkedHashMap<String, Object>> getDBCollectionQueryDataList (DataTable datatable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * DB연계 데이터 목록 총 갯수 조회
	 */
	public abstract int getDBCollectionQueryDataTotalRows (DataTable datatable) throws Exception;
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커트텀 쿼리 지정된 PK의 중복 데이터 수 (지정된 PK로 Table 생성이 가능한지 알아보기 위함) 
	 */
	public abstract int getCustomQueryPrimaryOverlapCount (DataTable datatable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * DataSource Close
	 */
	public abstract void dbConnectionClose() throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 쿼리 실행후, 실행한 쿼리문 조회
	 */
	public abstract String getQuery (String id, Object parameter) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커넥션된 데이터베이스의 실제 테이블 생성(pk 존재유)
	 */
	public abstract int datasetTableCreate(DataTable datatable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커넥션된 데이터베이스의 실제 테이블 생성(pk 존재무)
	 */
	public abstract int datasetDefaultTableCreate(DataTable datatable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 커넥션된 데이터 베이스 데이터 업데이트(pk 존재무)
	 */
	public abstract void datasetDataUpdate(DataTable datatable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 ROW 삭제
	 */
	public abstract void datasetDataDelete(DataTable datatable) throws Exception;
	
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 커넥션된 데이터 베이스 중복체크 (pk 존재무)
	 */
	public abstract int duplicateCheckTableName(Map info) throws Exception;
	
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 컬럼 정보 목록 조회 SQL에 접근하는 메소드 입니다.(페이징)
	 */
	public abstract List<LinkedHashMap<String, Object>> getRowData (DataTable dataTable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 컬럼 정보 목록 조회 SQL에 접근하는 메소드 입니다.(x)
	 */
	public abstract List<LinkedHashMap<String, Object>> getRowDataAll (DataTable dataTable) throws Exception;
	
	/**
	 * @author 최정우
	 * @since 2019.12.09
	 * 
	 * 데이터 셋의 컬럼 정보 목록 총 갯수 조회 메소드 입니다.
	 */
	public abstract int getRowDataTotalRows (DataTable dataTable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2022.1.12
	 * 
	 *  데이터 셋의 비우기 (데이터 셋 내용삭제)
	 */
	public abstract int emptyDataset (DataTable dataTable) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2022.1.20
	 * 
	 *  실제 생성된 컬럼 변경
	 */
	public abstract int  tableColumnChange (TableBasicInfo tableBasicInfo) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2022.1.20
	 * 
	 *  실제 생성된 테이블 PK처리
	 */
	public abstract int  changePrimaryKey (TableBasicInfo tableBasicInfo) throws Exception;
	
	/**
	 * @author 김성원
	 * @since 2022.1.20
	 * 
	 *  AI PK 생성 
	 */
	public abstract int  createAutoIncrement (TableBasicInfo tableBasicInfo) throws Exception;
	
	
	/**
	 * @author 김성원
	 * @since 2022.09.30
	 * 
	 *  테이블 명 변경
	 *   
	 */
	public abstract void updateTableMetaInfo(DataTable dataTable)  throws Exception;
}
