package com.takensoft.taken_bi_manager.common.connection.db.dao;



import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.ibatis.mapping.ParameterMapping;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.stereotype.Repository;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.takensoft.taken_bi_manager.common.connection.db.util.DBConnectionConfig;
import com.takensoft.taken_bi_manager.common.connection.db.vo.ConnectionDB;
import com.takensoft.taken_bi_manager.common.connection.db.vo.TableBasicInfo;
import com.takensoft.taken_bi_manager.common.util.StringUtil;
import com.takensoft.taken_bi_manager.data.vo.ColumnData;
import com.takensoft.taken_bi_manager.data.vo.DataTable;
import com.takensoft.taken_bi_manager.data.vo.Dataset;


/**
	 * @author 김성원
	 * @since 2024.01.09
 * 
 * Mariadb 접속 모듈에 관련된 SQL문에 접근하는 DAO Class입니다.
 */
@Repository
public class MariadbDAO extends DatabaseDAO {
	
	public MariadbDAO () {}
	
	public MariadbDAO (ConnectionDB db) throws Exception {
		this.createConnector(db);
	}
	
	private DBConnectionConfig dbUtil = null;
	
	public DBConnectionConfig getDbUtil() {
		return dbUtil;
	}
	public void setDbUtil(DBConnectionConfig dbUtil) {
		this.dbUtil = dbUtil;
	}

	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * SqlSessionTemplate 생성 메서드 입니다.
	 */
	@Override
	public void createConnector (ConnectionDB db) throws Exception {
		dbUtil = new DBConnectionConfig();
		dbUtil.mybatisSqlTemplateCreate(db);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 접속한 데이터베이스의 테이블 목록 조회 SQL에 접근하는 메소드 입니다.
	 */
	@Override
	public List<Dataset> getDBConnectionTableList(ConnectionDB db) throws Exception {
		try {
			return dbUtil.getTemplate().selectList("MariadbDAO.getDBConnectionTableList", db);
		} catch (Exception e) {
			return new ArrayList<Dataset>();
		}
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커넥션된 데이터베이스의 테이블의 컬럼 정보 조회 (DB 기준 데이터 타입, 데이터 크기 등)
	 */
	@Override
	public List<ColumnData> getDBConnectionTableColumnList(Dataset dataset) throws Exception {
		return dbUtil.getTemplate().selectList("MariadbDAO.getDBConnectionTableColumnList", dataset);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 특정 테이블의 데이터 목록 조회 SQL에 접근하는 메소드 입니다.
	 */
	@Override
	public List<LinkedHashMap<String, Object>> getDBConnectionTableDataList (DataTable datatable) throws Exception {
		return dbUtil.getTemplate().selectList("MariadbDAO.getDBConnectionTableDataList", datatable);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 특정 테이블의 데이터 총 개수 조회 SQL에 접근하는 메소드 입니다.
	 */
	@Override
	public int getDBConnectionTableDataTotalRows (DataTable datatable) throws Exception {
		return dbUtil.getTemplate().selectOne("MariadbDAO.getDBConnectionTableDataTotalRows", datatable);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커스텀 테이블 데이터 조회 (페이징사용 유무에 따라 페이징 처리) SQL에 접근하는 메소드 입니다.
	 */
	@Override
	public List<LinkedHashMap<String, Object>> getDBConnectionCustomQueryDataList (DataTable datatable) throws Exception {
		return dbUtil.getTemplate().selectList("MariadbDAO.getDBConnectionCustomQueryDataList", datatable);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커스텀 테이블 데이터 목록 총 갯수 조회 SQL에 접근하는 메소드 입니다.
	 */
	@Override
	public int getDBConnectionCustomQueryDataTotalRows (DataTable datatable) throws Exception {
		return dbUtil.getTemplate().selectOne("MariadbDAO.getDBConnectionCustomQueryDataTotalRows", datatable);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * DB연계 데이터 목록 조회 (초기 DB화 때에 등록한 컬럼만 가지고 옴)
	 */
	@Override
	public List<LinkedHashMap<String, Object>> getDBCollectionQueryDataList (DataTable datatable) throws Exception {
		return dbUtil.getTemplate().selectList("MariadbDAO.getDBCollectionQueryDataList", datatable);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * DB연계 데이터 목록 총 갯수 조회
	 */
	@Override
	public int getDBCollectionQueryDataTotalRows (DataTable datatable) throws Exception {
		return dbUtil.getTemplate().selectOne("MariadbDAO.getDBCollectionQueryDataTotalRows", datatable);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 커트텀 쿼리 지정된 PK의 중복 데이터 수 (지정된 PK로 Table 생성이 가능한지 알아보기 위함) 
	 */
	@Override
	public int getCustomQueryPrimaryOverlapCount(DataTable datatable) throws Exception {
		return dbUtil.getTemplate().selectOne("MariadbDAO.getCustomQueryPrimaryOverlapCount", datatable);
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * SqlSessionTemplate Close
	 */
	@Override
	public void dbConnectionClose() throws Exception {
		dbUtil.close();
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 쿼리 실행후, 실행한 쿼리문 조회
	 */
	@Override
	public String getQuery (String id, Object parameter) throws Exception {
		SqlSessionTemplate template = dbUtil.getTemplate();
		
		id = "OracleDAO." + id;
		
	    String sql = template.getConfiguration().getMappedStatement(id).getBoundSql(parameter).getSql();
	    List<ParameterMapping> paramMap = template.getConfiguration().getMappedStatement(id).getBoundSql(parameter).getParameterMappings();
	    
	    ObjectMapper mapper = new ObjectMapper();
	    String json = mapper.writeValueAsString(parameter);
	    HashMap<String, Object> params = mapper.readValue(json, new TypeReference<HashMap<String, Object>>(){});
	    for (ParameterMapping par : paramMap) {
	        String param = StringUtil.toString(params.get(par.getProperty()));
	        if(parameter == null)
	            sql = sql.replaceFirst("\\?", "NULL");
	        else
	            sql = sql.replaceFirst("\\?", "'" + param + "'");
	    }
	    System.out.println("");
	    System.out.println("---------------------- OracleDAO sql --------------------");
	    System.out.println(sql);
	    System.out.println("");
	    return sql;
	}

	@Override
	public int datasetTableCreate(DataTable datatable) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().update("MariadbDAO.datasetTableCreate", datatable);
	}

	@Override
	public int datasetDefaultTableCreate(DataTable datatable) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().update("MariadbDAO.datasetDefaultTableCreate", datatable);
	}

	@Override
	public void datasetDataUpdate(DataTable datatable) throws Exception {
		// TODO Auto-generated method stub
		dbUtil.getTemplate().update("MariadbDAO.datasetDataUpdate", datatable);
	}

	@Override
	public int duplicateCheckTableName(Map info) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().selectOne("MariadbDAO.duplicateCheckTableName", info);
	}

	@Override
	public List<LinkedHashMap<String, Object>> getRowData(DataTable dataTable) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().selectList("MariadbDAO.getRowData", dataTable);
	}

	@Override
	public List<LinkedHashMap<String, Object>> getRowDataAll(DataTable dataTable) throws Exception {
		return dbUtil.getTemplate().selectList("MariadbDAO.getRowDataAll", dataTable);		
	}

	@Override
	public int getRowDataTotalRows(DataTable dataTable) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().selectOne("MariadbDAO.getRowDataTotalRows", dataTable);		
	}

	@Override
	public int emptyDataset(DataTable dataTable) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().delete("MariadbDAO.emptyDataset", dataTable);		
	}

	@Override
	public int tableColumnChange(TableBasicInfo tableBasicInfo) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().update("MariadbDAO.tableColumnChange", tableBasicInfo);	
	}


	@Override
	public int changePrimaryKey(TableBasicInfo tableBasicInfo) throws Exception {
		// TODO Auto-generated method stub
		return dbUtil.getTemplate().update("MariadbDAO.changePromaryKey", tableBasicInfo);	
	}


	@Override
	public void datasetDataDelete(DataTable datatable) throws Exception {
		// TODO Auto-generated method stub
		dbUtil.getTemplate().delete("MariadbDAO.datasetDataDelete", datatable);	
	}

	@Override
	public int createAutoIncrement(TableBasicInfo tableBasicInfo) throws Exception {
		
		return dbUtil.getTemplate().update("MariadbDAO.createAutoIncrement", tableBasicInfo);	

	}

	@Override
	public int executeCustomQuery(DataTable datatable) throws Exception {
	
		return dbUtil.getTemplate().update("MariadbDAO.executeCustomQuery", datatable);	
	}


	@Override
	public void updateTableMetaInfo(DataTable dataTable) throws Exception {
		// TODO Auto-generated method stub
		
	}
}
