package com.takensoft.taken_bi_manager.common.connection.db.util;

import java.util.Arrays;
import java.util.List;

import com.takensoft.taken_bi_manager.common.connection.db.vo.DataType;
import com.takensoft.taken_bi_manager.common.vo.SystemCode.DatabaseType;
import com.takensoft.taken_bi_manager.data.vo.ColumnData;

/**
 * @author 김성원
 * @since 2024.01.02
 * 
 * 데이터 타입별로 DB -> JAVA ,  JAVA -> DB 변경을 위한 인터페이스 입니다.
 */
public class DataTypeUtil {

	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 데이터베이스 자료타입을 자바형태로 변환
	 */
	public static DataType convertDataTypeDbtoJava (ColumnData column, DatabaseType databaseType ) {
    	
		 DataType dataType = null;
		 
		if(databaseType.equals(DatabaseType.MARIADB) || databaseType.equals(DatabaseType.MYSQL)){
			dataType = mariadbDataConvertDBtoJava(column);
		}else {
			dataType = postgresqlDbDataConvertDBtoJava(column);
		}
    	
    	return dataType;	    	
    }
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 자바 자료형태를 DB 타입으로 변환 
	 */
	public static DataType convertDataTypeJavatoDB (ColumnData column, DatabaseType databaseType ) {
    	
    	
    	return null;	    	
    }
	
	public static List<ColumnData> convertDataTypeDbtoJavaList(List<ColumnData> ColumnDatas , DatabaseType databaseType){
		
		for(ColumnData item : ColumnDatas) {
			item.setDataTy(convertDataTypeDbtoJava(item,databaseType));
		}
		
		
		return ColumnDatas;		
	}
	
	 
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 마리아 및 myslq 데이터베이스 자료타입을 자바형태로 변환
	 */
	 private static DataType mariadbDataConvertDBtoJava(ColumnData column) {
		 
		 DataType dataType = null;
		 
		 // 스트링 타입 변환 
		 if(Arrays.asList("TEXT", "VARCHAR", "TINYTEXT", "LONGTEXT","ENUM","SET","CHAR","LONGBLOB","BLOB","TINYBLOB","VARBINARY","BINARY","CHAR").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.STRING;
		 }
		 // boolean 타이 변환 
		 else  if(Arrays.asList("BIT").contains(column.getDbDataType().toLowerCase().trim())) {
			 dataType = DataType.INT;
		 }
		 // Integer 타이 변환 
		 else  if(Arrays.asList("TINYINT", "SMALLINT").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.INT;
		 }
		// Long 타이 변환 
		 else  if(Arrays.asList("MEDIUMINT", "INT","BIGINT").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.INT;
		 }
		 // float 타이 변환 
		 else  if(Arrays.asList("FLOAT").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.FLOAT;
		 }
		 // double 타이 변환 
		 else  if(Arrays.asList("DOUBLE").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.DOUBLE;
		 }
		 // DECIMAL  타이 변환 
		 else  if(Arrays.asList("DECIMAL").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.DECIMAL;
		 }
		 // DATE  타이 변환 
		 else  if(Arrays.asList("DATE").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.DATE;
		 } 
		 // TIMESTAMP  타이 변환 
		 else  if(Arrays.asList("DATETIME,TIMESTAMP,TIME").contains(column.getDbDataType().toUpperCase().trim())) {
			 dataType = DataType.TIMESTAMP;
		 } 		
		 
		 return dataType;
	 }
	 
	 /**
		 * @author 김성원
		 * @since 2024.01.09
		 * 
		 * 마리아 및 myslq 데이터베이스 자료타입을 자바형태로 변환
		 */
		 private static DataType postgresqlDbDataConvertDBtoJava(ColumnData column) {
			 
			 DataType dataType = null;
			 
			 // 스트링 타입 변환 
			 if(Arrays.asList("character varying", "character","text").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.STRING;
			 }
			 // boolean 타이 변환 
			 else  if(Arrays.asList("bit").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.BOOL;
			 }
			 // Integer 타이 변환 
			 else  if(Arrays.asList("integer","smallint","serial").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.INT;
			 }
			 // Long 타이 변환 
			 else  if(Arrays.asList("bigint", "bigserial").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.LONG;
			 }			
			 // float 타이 변환 
			 else  if(Arrays.asList("real").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.FLOAT;
			 }
			 // double 타이 변환 
			 else  if(Arrays.asList("double precision","numeric").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.DOUBLE;
			 }			
			 // DATE  타이 변환 
			 else  if(Arrays.asList("date","time without time zone").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.DATE;
			 } 
			 // TIMESTAMP  타이 변환 
			 else  if(Arrays.asList("time","timestamp","timestamp with time zone","timestamp without time zone", "time with time zone").contains(column.getDbDataType().toLowerCase().trim())) {
				 dataType = DataType.TIMESTAMP;
			 }else {
				 dataType = DataType.STRING;
			 }
			 
			 return dataType;
		 }
	
}
