package com.takensoft.taken_bi_manager.common.connection.db.vo;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import com.takensoft.taken_bi_manager.common.util.CommonUtil;
import com.takensoft.taken_bi_manager.common.util.StringUtil;

/**
 * @author 김성원
 * @since 2024.01.02
 * 
 * 데이터 타입을 정의한 상수 Class 입니다.
 * 상수명(java데이터타입, 초기값, mysql데이터타입, display데이터타입
 */

public enum DataType {
	NULL(null, null)
	, BYTE(Byte.class, 0)
	, BOOL(Boolean.class, false)
	, CHAR(Character.class, '\u0000')
	, STRING(String.class, "")
	, SHORT(Short.class, 0)
	, INT(Integer.class, 0)
	, DECIMAL(BigDecimal.class,0)
	, LONG(Long.class, 0)
	, FLOAT(Float.class, 0.0)
	, DOUBLE(Double.class, 0.0)
	, DATE(Date.class, "")
	, DATETIME(Date.class, "")
	, TIMESTAMP(Timestamp.class, "")
	, ENTER(Character.class,'\n');

	/**
	 * enum(열거형데이터)의 생성자 
	 * 1. 상수에 나열된 parameter와 형태가 똑같이 만들어줘야함.
	 * 2. 접근제한자는 private로만 다른 접근제한자는 허용하지 않음
	 */
	private DataType (Class<?> javaType, Object iniValue) {
        this.javaType = javaType;
        this.initValue = iniValue;
    }
	
	/**
	 * java데이터타입
	 */
	final private Class<?> javaType;
	
	/**
	 * 초기값
	 */
	final private Object initValue;
	
    public Class<?> getJavaType() {
        return javaType;
    }
    public Object getInitValue() {
        return initValue;
    }
    	 
    
    /**
	 * 데이터 타입 검사, DataType 조회
	 */
    public static Map<DataType, String> getDataTypeList () {
    	Map<DataType, String> info = new LinkedHashMap<DataType, String>();
    	info.put(STRING, "문자열");
    	info.put(LONG, "정수");
    	info.put(DOUBLE, "실수");
    	info.put(DATE, "날짜");
    	info.put(DATETIME,"날짜,시간");
    	return info;
	}
    
    
    /**
	 * 데이터 타입 검사, DataType 조회
	 */
    public static List<DataType> getDataTypeArray () {
    	List<DataType> info = new ArrayList<DataType>();
    	info.add(STRING);
    	info.add(LONG);
    	info.add(DOUBLE);
    	info.add(DATE);
    	info.add(DATETIME);
    	info.add(BOOL);
    	return info;
	}
    
    /**
	 * 데이터 타입 검사, DataType 조회
	 */
    public static DataType getDataType (String text) {
    	/*
    	 * 2021-11-02 김혜민 dataType 전화번호, 날짜 정규식 수정
    	 * 
    	 * 
    	 * 전화번호 정규식*/
    	String regExp = "^\\d{2,3}-\\d{3,4}-\\d{4}$";
    	String regDateTime = "\\d{4}-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01]) (0[1-9]|1[0-9]|2[0-4]):(0[1-9]|[1-5][0-9]):(0[1-9]|[1-5][0-9])";
    	String regDate = "\\d{4}-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])";
    			
		if(StringUtil.isEmpty(text)) {
			return NULL;
		} else if(CommonUtil.isLong(text)) {
			/*
			 * 숫자로 변환 가능 하지만 앞에 '0'이 붙어 있으면 문자열로 봄
			 * ex) 0102 -> String, 1102 -> Long
			 */
			if (text.length() > 1 && text.indexOf("0") == 0) {
				return STRING;
			} else {
				return LONG;
			}
		}else if(CommonUtil.isDouble(text)) {
			
			return DOUBLE;
			
		} else if (CommonUtil.isDate(text)) {
			/*
	    	 * 2021-11-02 김혜민 dataType 전화번호, 날짜 정규식 수정
	    	 * 
	    	 * 
	    	 * 전화번호 정규식*/
	    	boolean expResult = Pattern.matches(regExp, text);
	    	boolean dateTimeResult = Pattern.matches(regDateTime, text);
	    	boolean dateResult = Pattern.matches(regDate, text);
	    	
			if(expResult) {
				return STRING;
			}else if(dateResult) {				
				return DATE;
			}else {
				return DATETIME;
			}
			
		} else {
			return STRING;
		}
	}
    
    /**
	 * 데이터 타입 검사, dbDataType 조회
	 */
    public static String convertDbDataType (DataType datatype, Integer size) {
    	final int MAX_VARCHAR_SIZE = 765;
    	
    	if (datatype == STRING) {
    		if (size == null || size <= MAX_VARCHAR_SIZE) {
    			return "varchar";
    		} else {
    			return "text";
    		}
    	} else if (datatype == DATE) {
    		return "date";
    	} else if (datatype == DATETIME){
    		return "datetime";
    	} else if (datatype == LONG) {
    		return "bigint";
    	} else if (datatype == DOUBLE) {
    		return "double";
    	} else {
    		return "varchar";
    	}
    }
    
    /**
	 * 데이터 타입 검사, dbDataType -> javaType 조회
	 */
    public static DataType convertDataTypeDbtoJava (DataType datatype, Integer size) {
    	
    	
    	return null;	    	
    }
    
    
    /**
	 * 데이터 타입 형태로 값 변경
	 */
	public static Object parse(DataType dataType, Object value) {
		if (CommonUtil.isNull(value) == true) {
			return null;
		}
		
		Class<?> javaType = dataType.getJavaType();
		try {
			if (javaType.getSimpleName().equals("Byte")) {
				return Byte.parseByte(value.toString());
			} else if (javaType.getSimpleName().equals("Boolean")) {
				return Boolean.parseBoolean(value.toString());
			} else if (javaType.getSimpleName().equals("Character")) {
				return value.toString().toCharArray();
			} else if (javaType.getSimpleName().equals("String")) {
				return value.toString();
			} else if (javaType.getSimpleName().equals("Short")) {
				//Short.parseShort(value.toString().replaceAll("[^0-9]",""));
				return Short.parseShort(value.toString());
			} else if (javaType.getSimpleName().equals("Integer")) {
				//Integer.parseInt(value.toString().replaceAll("[^0-9]",""));
				return Integer.parseInt(value.toString());
			} else if (javaType.getSimpleName().equals("Long")) {
				//Long.parseLong(value.toString().replaceAll("[^0-9]",""));
				return Long.parseLong(value.toString());
			} else if (javaType.getSimpleName().equals("Float")) {
				//Float.parseFloat(value.toString().replaceAll("[^0-9]",""));
				return Float.parseFloat(value.toString());
			} else if (javaType.getSimpleName().equals("Double")) {
				//Double.parseDouble(value.toString().replaceAll("[^0-9]",""));
				return Double.parseDouble(value.toString());
			} else {
				return value;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return dataType.getInitValue();
		}
	}
	
	/**
	 * 데이터 타입 형태로 값 변경
	 */
	public static boolean parseCheck (DataType dataType, Object value) {
		if (CommonUtil.isNull(value) == true) {
			return false;
		}
		
		Class<?> javaType = dataType.getJavaType();
		try {
			if (javaType.getSimpleName().equals("Byte")) {
				Byte.parseByte(value.toString());
			} else if (javaType.getSimpleName().equals("Boolean")) {
				Boolean.parseBoolean(value.toString());
			} else if (javaType.getSimpleName().equals("Character")) {
				value.toString().toCharArray();
			} else if (javaType.getSimpleName().equals("String")) {
				value.toString();
			} else if (javaType.getSimpleName().equals("Short")) {
				//Short.parseShort(value.toString().replaceAll("[^0-9]",""));
				Short.parseShort(value.toString());
			} else if (javaType.getSimpleName().equals("Integer")) {
				//Integer.parseInt(value.toString().replaceAll("[^0-9]",""));
				Integer.parseInt(value.toString());
			} else if (javaType.getSimpleName().equals("Long")) {
				//Long.parseLong(value.toString().replaceAll("[^0-9]",""));
				Long.parseLong(value.toString());
			} else if (javaType.getSimpleName().equals("Float")) {
				//Float.parseFloat(value.toString().replaceAll("[^0-9]",""));
				Float.parseFloat(value.toString());
			} else if (javaType.getSimpleName().equals("Double")) {
				//Double.parseDouble(value.toString().replaceAll("[^0-9]",""));
				Double.parseDouble(value.toString());
			}
			
			return true;
		} catch (Exception e) {
			//e.printStackTrace();
			return false;
		}
	}
	
	
}

