package com.takensoft.taken_bi_manager.user.member.servie.impl;

import java.util.HashMap;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.takensoft.taken_bi_manager.common.util.AuthUtil;
import com.takensoft.taken_bi_manager.common.util.CommonUtil;
import com.takensoft.taken_bi_manager.common.util.CryptoUtil;
import com.takensoft.taken_bi_manager.common.util.SesssionEventListener;
import com.takensoft.taken_bi_manager.common.util.StringUtil;
import com.takensoft.taken_bi_manager.common.vo.CustomeResultMap;
import com.takensoft.taken_bi_manager.user.member.dao.LoginDAO;
import com.takensoft.taken_bi_manager.user.member.dao.MemberDAO;
import com.takensoft.taken_bi_manager.user.member.servie.LoginService;
import com.takensoft.taken_bi_manager.user.member.vo.Member;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;


/**
 * @author 김성원
 * @since 2024.01.10
 * 
 * 로그인 관련 처리 서비스 클래스
 */
@Service
public class LoginServiceImpl implements LoginService {

	@Autowired
    private MemberDAO memberDAO;
	
	@Autowired
    private LoginDAO loginDAO;
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.09
	 * 
	 * 유저로그인 처리(userId, userPassword)
	 */
	@Override
	public CustomeResultMap userLogin(Member member) throws Exception {			
		
		// 결과맵 생성
		CustomeResultMap resultMap = new CustomeResultMap();
		
		//사용자 정보 조회 
		Member memebrInfo = memberDAO.getMemnberById(member.getUserId());
		
		// 사용자 정보 있음
		if(memebrInfo != null && !StringUtil.isEmpty(memebrInfo.getUserId())){
					
			
			// 사용자 정보 복호화
			memebrInfo.dataDecoder();
			
			if(memebrInfo.isLockAt()) {
				resultMap.getCheckMessage().setError("계정이 잠겨있습니다. 관리자에게 문의하세요");	
				return resultMap;
			}
			
			//비밀번호 비교 성공
			if(CryptoUtil.passwordMatch(member.getUserPassword(), memebrInfo.getUserPassword())){
				
				/** 로그인 처리이후, 세션처리 (시작) **/
	            //세션 무효화 처리
				userLogout();
				
				//현재 로그인할 userId와 동일한 로그인 session을 가진 session ID 목록 조회
	            List<String> sessionIds = SesssionEventListener.duplicationLoginSessionIdSelectListByUserId(member.getUserId());
	           
	            /**
	             * 현재 로그인 요청한 userId와 동일한 접속 session이 1명 이상 있고,
	             * 중복 로그인 가능 유무가 true이면 -> 중복 로그인 처리 진행
	             * 중복 로그인 가능 유무가 false이면 -> 기존 로그인 유저를 session에서 제거
	             */
	            if (sessionIds.size() > 0 && AuthUtil.IS_POSSIBLE_DUPLICATION_LOGIN == false)  {
	                //기존 로그인 유저 session에서 제거
	                SesssionEventListener.duplicationLoginSessionDeleteByUserId(member.getUserId());
	            }
				
				//현재 Http Request 객체 가지고 오기
		        HttpServletRequest request = CommonUtil.getHttpServletRequest();
				
		        //기존 session이 존재하더라도 제거하고 새로운 세션 생성
	            HttpSession session = request.getSession(true);
	            
	            //세션 timeout(30분)
	            session.setMaxInactiveInterval(60 * 30); 
	            
	            // session 저장 정보 생성
	            HashMap<String, Object> LoginUserInfo = new HashMap<>();	            
	            LoginUserInfo.put("user_id", memebrInfo.getUserId());							//유저아이디
	            LoginUserInfo.put("user_name", memebrInfo.getUserNm());							//유저이름
	            LoginUserInfo.put("user_auth", memebrInfo.getAuthList());     					//권한리스트 
	            LoginUserInfo.put("dept_name", memebrInfo.getDeptNm());      					//부서명
	            
	            // session 생성
	            session.setAttribute(AuthUtil.LOGIN_USER_SESSION, LoginUserInfo);    
	            resultMap.getCheckMessage().setMessage( memebrInfo.getUserId() + "님 환영합니다.");	            
	            
	            // 성공로직 아래에 만들기 
	            loginDAO.loginSuccess(member);  
	            
			}
			// 실패
			else {	
				// 로그인 비밀번호 실패카운트
				memebrInfo.setLoginFailrCnt(memebrInfo.getLoginFailrCnt() + 1);
				
				if(memebrInfo.getLoginFailrCnt() ==5) {
					memebrInfo.setLockAt(true);
				}
				
				resultMap.getCheckMessage().setError("비밀번호 불일치 "+memebrInfo.getLoginFailrCnt()+"회, 5회 실패시 계정이 잠깁니다." );						
				// 비밀번호 실패 업데이트 로직 	
				 loginDAO.passowrdFail(memebrInfo);  
			}
			
							
		}else {
			resultMap.getCheckMessage().setError("일치하는 정보가 없습니다. \\r\\n사용자 정보를 확인해 주세요");		
		}				
		
		return resultMap;
	}

	/**
     * @author 김성원
	 * @since 2024.01.10
     *
     * 세션 무효화 (로그아웃 처리)
     */
	@Override
	public boolean userLogout() throws Exception {

		//현재 Http Request 객체 가지고 오기
        HttpServletRequest request = CommonUtil.getHttpServletRequest();
        //기존 session이 존재하면 session값을 받아오고, 존재하지 않으면 null
        HttpSession session = request.getSession(false);
        //기존 session이 존재하면 -> session 삭제
        if (session != null) {
            //session 삭제
            session.invalidate();
        }

		return false;
	}
}
